@@ -4,8 +4,8 @@
     Porthole command line entry Dialog
     Shows information about Porthole
 
-    Copyright (C) 2003 - 2004 Fredrik Arnerup, Daniel G. Taylor
-                                Brian Dolbec and William F. Wheeler
+    Copyright (C) 2003 - 2005 Fredrik Arnerup, Daniel G. Taylor,
+    Brian Dolbec and William F. Wheeler
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -23,25 +23,29 @@
 '''
 
 import gtk, gtk.glade
-from utils import dprint, load_web_page
+from utils import dprint
+from loaders import load_web_page
 from version import version
 from gettext import gettext as _
 
 class RunDialog:
     """Class to hold run dialog and functionality."""
 
-    def __init__(self, prefs, call_back):
+    def __init__(self, prefs, call_back, run_anyway=False):
         # setup glade
-        self.gladefile = prefs.DATA_PATH + "porthole.glade"
+        self.gladefile = prefs.DATA_PATH + prefs.use_gladefile
         self.wtree = gtk.glade.XML(self.gladefile, "run_dialog")
         # register callbacks
         callbacks = {"on_help" : self.help,
-		     "on_execute" : self.execute,
-                     "on_cancel" : self.cancel}
+                     "on_execute" : self.execute,
+                     "on_cancel" : self.cancel,
+                     "on_comboboxentry1_changed" : self.command_changed
+                    }
         self.wtree.signal_autoconnect(callbacks)
         self.command = None
         self.call_back = call_back
         self.prefs = prefs
+        self.run_anyway = run_anyway
         if self.prefs:
             #dprint("COMMAND: self.prefs == True")
             self.history = self.prefs.run_dialog.history
@@ -51,15 +55,22 @@ class RunDialog:
                             "ACCEPT_KEYWORDS='~x86' emerge ",
                             "USE=' ' emerge ",
                             "ACCEPT_KEYWORDS='~x86' USE=' ' emerge"]
-        dprint("COMMAND: self.history:")
-        dprint(self.history)
+        #dprint("COMMAND: self.history:")
+        #dprint(self.history)
         self.window = self.wtree.get_widget("run_dialog")
-        self.combo = self.wtree.get_widget("combo")
-        self.entry = self.wtree.get_widget("combo-entry")
-        self.list = self.wtree.get_widget("combo-list")
-        #self.window.set_title("Porthole")
-        if len(self.history):
-            self.combo.set_popdown_strings(self.history)
+        self.combo = self.wtree.get_widget("comboboxentry1")
+        self.entry = self.combo.child
+        #self.list = self.wtree.get_widget("combo-list")
+        # Build a formatted combo list from the versioninfo list 
+        self.comboList = gtk.ListStore(str)
+        index = 0
+        for x in self.history:
+            # Set the combo list
+            self.comboList.append([x])
+        
+        # Set the comboboxentry to the new model
+        self.combo.set_model(self.comboList)
+        self.combo.set_text_column(0)
         self.entry.connect("activate", self.activate, self.command)
         if self.prefs:
             self.window.resize(self.prefs.run_dialog.width, 
@@ -74,7 +85,7 @@ class RunDialog:
         self.command = self.entry.get_text()
         if self.command:
             #dprint("COMMAND: activated: %s" %self.command)
-            self.call_back(_("command line entry"), self.command)
+            self.call_back(_("command line entry"), self.command, self.run_anyway)
             self.history_add()
         self.cancel(None)
         
@@ -83,7 +94,7 @@ class RunDialog:
         self.command = self.entry.get_text()
         if self.command:
             #dprint("COMMAND: execute: %s" %self.command)
-            self.call_back(_("command line entry"), self.command)
+            self.call_back(_("command line entry"), self.command, self.run_anyway)
             self.history_add()
         self.cancel(None)
 
@@ -93,7 +104,7 @@ class RunDialog:
         
     def help(self, widget):
         """ Display help file with web browser """
-        load_web_page('file://' + self.prefs.DATA_PATH + 'help/custcmd.html')
+        load_web_page('file://' + self.prefs.DATA_PATH + 'help/custcmd.html', self.prefs)
 
 
     def on_size_request(self, window, gbox):
@@ -120,4 +131,12 @@ class RunDialog:
             dprint(self.history)
         self.prefs.run_dialog.history = self.history
         return
+
+    def command_changed(self,widget):
+        """Updates the gtk.Entry with the history item selected"""
+        dprint("COMMAND: changing entry item")
+        return # not needed at this time
+        model = widget.get_model()
+        iter = widget.get_active_iter()
+        selection = model.get_value(iter, 0)
         
