@@ -4,8 +4,8 @@
     Porthole Utils Package
     Holds common functions for Porthole
 
-    Copyright (C) 2003 - 2004 Fredrik Arnerup and Daniel G. Taylor
-                              Brian Dolbec and Wm. F. Wheeler
+    Copyright (C) 2003 - 2005 Fredrik Arnerup, Daniel G. Taylor
+    Brian Dolbec, Wm. F. Wheeler, Tommy Iorns
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -31,8 +31,10 @@ import string
 import sre
 import datetime
 
+#import portagelib
+
 from sys import stderr
-from version import version
+from version import *
 from xmlmgr import XMLManager, XMLManagerError
 from gettext import gettext as _
 
@@ -42,18 +44,6 @@ import os
 import grp
 import pwd, cPickle
 
-# if using gnome, see if we can import it
-try:
-    import gnome
-    import gnome.vfs
-except ImportError:
-    # no gnome module, use the standard webbrowser module
-    try:
-        import webbrowser
-    except ImportError:
-        print >>stderr, ('Module "webbrowser" not found. '
-                     'You will not be able to open web pages.')
-
 def dprint(message):
     """Print debug message if debug is true."""
     if debug:
@@ -67,42 +57,11 @@ def dsave(name, item = None):
         home = pwd.getpwuid(os.getuid())[5]
         # pickle it baby, yeah!
         cPickle.dump(item, open(home + "/.porthole/" + name, "w"))
-        
-
-def load_web_page(name):
-    """Try to load a web page in the default browser"""
-    dprint("UTILS: load_web_page()")
-    browser = web_page(name)
-    browser.start()
-    return
-
-class web_page(threading.Thread):
-    """Try to load a web page in the default browser"""
-    def __init__(self, name):
-        dprint("UTILS: web_page.__init__()")
-        threading.Thread.__init__(self)
-        self.name = name
-        self.setDaemon(1)  # quit even if this thread is still running
-        
-    def run(self):
-        dprint("UTILS: web_page.run()")
-        if self.name == '' or self.name == None:
-            return
-        try:
-            gnome.url_show(self.name)
-        except:
-            dprint("Gnome failed trying to open: %s" %self.name)
-            try:
-                webbrowser.open(self.name)
-            except:
-                dprint("webbrowser failed trying to open: %s  -- giving up" %self.name)
-                pass
-        dprint("Browser call_completed for: %s" %self.name)
 
 def get_icon_for_package(package):
     """Return an icon for a package"""
     # if it's installed, find out if it can be upgraded
-    if package.is_installed:
+    if package and package.get_installed():
         icon = gtk.STOCK_YES
     else:
         # just put the STOCK_NO icon
@@ -110,6 +69,19 @@ def get_icon_for_package(package):
         icon = '' # gtk.STOCK_NO
     return icon       
 
+def get_icon_for_upgrade_package(package, view_prefs):
+    """Return an icon and foreground text color for a package"""
+    if not package:
+        return '', 'blue'
+    #  find out if it can be upgraded
+    if package.is_upgradable() == 1:  # 1 for only upgrades (no downgrades)
+        icon = gtk.STOCK_GO_UP
+        color = view_prefs.upgradable_fg
+    else: # it's a downgrade
+        icon = gtk.STOCK_GO_DOWN
+        color = view_prefs.downgradable_fg
+    return icon, color      
+
 def is_root():
     """Returns true if process runs as root."""
     return os.geteuid() == 0
@@ -121,10 +93,36 @@ def read_access():
     # Note: you don't have to be a member of portage to read the database,
     # but portage caching will not work
     portage = 250  # is portage guaranteed to be 250?
+    # answer: portage.portage_gid
     try: portage = grp.getgrnam("portage")[2]
     except: pass
     return write_access() or (portage in (os.getgroups() + [os.getegid()]))
 
+sudo_x_ok = os.access('/usr/bin/sudo', os.X_OK)
+
+def can_sudo():
+    """ return True if /usr/bin/sudo exists and is executable """
+    return sudo_x_ok
+    #return False # for testing
+
+gksudo_x_ok = os.access('/usr/bin/gksudo', os.X_OK)
+gksu_x_ok = os.access('/usr/bin/gksu', os.X_OK)
+gnomesu_x_ok = os.access('/usr/bin/gnomesu', os.X_OK)
+kdesu_x_ok = os.access('/usr/bin/kdesu', os.X_OK)
+
+def can_gksu(specific=None):
+    if not specific:
+        return gksudo_x_ok or gksu_x_ok or gnomesu_x_ok or kdesu_x_ok
+    if specific == 'gksudo':
+        return gksudo_x_ok
+    if specific == 'gksu':
+        return gksu_x_ok
+    if specific == 'gnomesu':
+        return gnomesu_x_ok
+    if specific == 'kdesu':
+        return kdesu_x_ok
+    return None
+
 def get_treeview_selection( treeview, num = None):
         """Get the value of whatever is selected in a treeview,
         num is the column, if num is nothing, the iter is returned"""
@@ -148,7 +146,7 @@ def environment():
         env["FEATURES"] += ", notitles"
     else:
         env ["FEATURES"] = "notitles"
-    env["NOCOLOR"] = "true"
+    #env["NOCOLOR"] = "true"
     #dprint(env)
     return env
 
@@ -177,9 +175,9 @@ class YesNoDialog(CommonDialog):
     def __init__(self, title, parent = None,
                  message = None, callback = None):
         CommonDialog.__init__(self, title, parent, message,
-                                           callback, "_Yes")
+                                           callback, _("_Yes"))
         # add "No" button
-        self.add_button("_No", 1)
+        self.add_button(_("_No"), 1)
         
 
 class SingleButtonDialog(CommonDialog):
@@ -194,7 +192,14 @@ class SingleButtonDialog(CommonDialog):
             self.progbar.show()
             self.vbox.add(self.progbar)
 
-class EmergeOptions:
+class OptionsClass:
+    """Blank class to hold options"""
+    
+    def __repr__(self):
+        """Return options as a string"""
+        return self.__dict__.__repr__()
+
+class EmergeOptions(OptionsClass):
     """ Holds common emerge options """
     def __init__(self):
         # let's get some saved values in here!
@@ -210,20 +215,14 @@ class EmergeOptions:
         if self.pretend:   opt_string += '--pretend '
         if self.fetch:     opt_string += '--fetchonly '
         if self.verbose:   opt_string += '--verbose '
-        if self.upgradeonly: opt_string += '--upgradeonly ' 
+        # upgradeonly does not have the desired effect when passed to emerge.
+        #if self.upgradeonly: opt_string += '--upgradeonly '
         if self.nospinner: opt_string += '--nospinner '
         return opt_string
 
-class WindowPreferences:
-    """ Holds preferences for a window """
-    def __init__(self, width = 0, height = 0):
-        self.width = width      # width
-        self.height = height    # height
-
-
 class PortholePreferences:
     """ Holds all of Porthole's user configurable preferences """
-    def __init__(self):
+    def __init__(self, New_prefs = None):
 
         # establish path & name of user prefs file
 
@@ -249,281 +248,182 @@ class PortholePreferences:
         # set pref to default value.  The beauty of this is: older versions
         # of the prefs files are still compatible and even if the user has
         # no prefs file, it still works!
-
-        # Main window settings
-
-        try:
-           width = dom.getitem('/window/main/width')
-        except XMLManagerError:
-           width = 200   # Default value
-        try:
-           height = dom.getitem('/window/main/height')
-        except XMLManagerError:
-           height = 350   # Default value
-        self.main = WindowPreferences(width, height)
-        try:
-           hpane = dom.getitem('/window/main/hpane')
-        except XMLManagerError:
-           hpane = 180   # Default value
-        self.main.hpane = hpane
-        dprint("UTILS: __init__() hpane: %d" %self.main.hpane)
-        try:
-           vpane = dom.getitem('/window/main/vpane')
-        except XMLManagerError:
-           vpane = 125   # Default value
-        self.main.vpane = vpane
-        try:
-           search_desc = dom.getitem('/window/main/search_desc')
-        except XMLManagerError:
-           search_desc = False   # Default value
-        self.main.search_desc = search_desc
-        try:
-           show_nag_dialog = dom.getitem('/window/main/show_nag_dialog')
-        except XMLManagerError:
-           show_nag_dialog = True   # Default value
-        self.main.show_nag_dialog = show_nag_dialog
-
-        # Process window settings
-
-        try:
-           width = dom.getitem('/window/process/width')
-        except XMLManagerError:
-           width = 300   # Default value
-        try:
-           height = dom.getitem('/window/process/height')
-        except XMLManagerError:
-           height = 350   # Default value
-        self.process = WindowPreferences(width, height)
-        try:
-           width_verbose = dom.getitem('/window/process/width_verbose')
-        except XMLManagerError:
-           width_verbose = 500   # Default value
-        self.process.width_verbose = width_verbose
-
-        # Terminal window settings
-
-        try:
-           width = dom.getitem('/window/terminal/width')
-        except XMLManagerError:
-           width = 300   # Default value
-        try:
-           height = dom.getitem('/window/terminal/height')
-        except XMLManagerError:
-           height = 350   # Default value
-        self.terminal = WindowPreferences(width, height)
-        try:
-           width_verbose = dom.getitem('/window/terminal/width_verbose')
-        except XMLManagerError:
-           width_verbose = 500   # Default value
-        self.terminal.width_verbose = width_verbose
+        
+        preflist = {}
+        
+        preflist['main'] = [ \
+            ['width', 200],
+            ['height', 350],
+            ['xpos', False],
+            ['ypos', False],
+            ['hpane', 180],
+            ['vpane', 125],
+            ['maximized', False],
+            ['search_desc', False],
+            ['show_nag_dialog', True]
+        ]
+        
+        preflist['process'] = [ \
+            ['width', 300],
+            ['height', 350],
+            ['width_verbose', 500]
+        ]
+        
+        preflist['terminal'] = [ \
+            ['width', 300],
+            ['height', 350],
+            ['width_verbose', 500],
+            ['all_tabs_use_custom_colors', False],
+            ['font', None]
+        ]
+        
+        history = ["",
+                   "emerge ",
+                   "ACCEPT_KEYWORDS='~x86' emerge ",
+                   "USE=' ' emerge ",
+                   "ACCEPT_KEYWORDS='~x86' USE=' ' emerge ",
+                   "emerge --help"]
+        # default_history = length of the history items to always remain
+        # at the start of the popdown history list & set above when history is set
+        # history_length = Default value for maximum nuber of retained history items
+        
+        preflist['run_dialog'] = [ \
+            ['width', 400],
+            ['height', 120],
+            ['history', history],
+            ['default_history', len(history)],
+            ['history_length', 10]
+        ]
+        
+        for window_name in preflist.keys():
+            setattr(self, window_name, OptionsClass()) # construct self.main etc.
+            for pref_name, default_value in preflist[window_name]:
+                try:
+                    value = dom.getitem('/'.join(['/window',window_name,pref_name]))
+                    # (e.g. '/window/main/width')
+                except XMLManagerError:
+                    value = default_value
+                setattr(getattr(self, window_name), pref_name, value) # set self.main.width etc
         
         # Formatting tags for the terminal window tabs.  
         # Note: normal font weight = 400 (pango.WEIGHT_NORMAL),
         #       bold = 700 (pango.WEIGHT_BOLD)
         # Note: all colors are in hex for future color editor;
         #       '' means use default color.
-
-        # Caution tag
-
+        
         self.TAG_DICT = {}
-        try:
-           forecolor = dom.getitem('/window/terminal/tag/caution/forecolor')
-        except XMLManagerError:
-           forecolor = ''  # Default value
-        try:
-           backcolor = dom.getitem('/window/terminal/tag/caution/backcolor')
-        except XMLManagerError:
-           backcolor = '#ff14b4'  # Default value
-        try:
-           fontweight = dom.getitem('/window/terminal/tag/caution/fontweight')
-        except XMLManagerError:
-           fontweight = 400  # Default value
-        self.TAG_DICT['caution'] = [forecolor, backcolor, fontweight]
-
-        # Command tag
-
-        try:
-           forecolor = dom.getitem('/window/terminal/tag/command/forecolor')
-        except XMLManagerError:
-           forecolor = '#ffffff'  # Default value
-        try:
-           backcolor = dom.getitem('/window/terminal/tag/command/backcolor')
-        except XMLManagerError:
-           backcolor = '#000080'  # Default value
-        try:
-           fontweight = dom.getitem('/window/terminal/tag/command/fontweight')
-        except XMLManagerError:
-           fontweight = 700  # Default value
-        self.TAG_DICT['command'] = [forecolor, backcolor, fontweight]
-
-        # Emerge tag
-
-        try:
-           forecolor = dom.getitem('/window/terminal/tag/emerge/forecolor')
-        except XMLManagerError:
-           forecolor = ''  # Default value
-        try:
-           backcolor = dom.getitem('/window/terminal/tag/emerge/backcolor')
-        except XMLManagerError:
-           backcolor = '#90ee90'  # Default value
-        try:
-           fontweight = dom.getitem('/window/terminal/tag/emerge/fontweight')
-        except XMLManagerError:
-           fontweight = 700  # Default value
-        self.TAG_DICT['emerge'] = [forecolor, backcolor, fontweight]
-
-        # Error tag
-
-        try:
-           forecolor = dom.getitem('/window/terminal/tag/error/forecolor')
-        except XMLManagerError:
-           forecolor = '#faf0e6'  # Default value
-        try:
-           backcolor = dom.getitem('/window/terminal/tag/error/backcolor')
-        except XMLManagerError:
-           backcolor = '#ff0000'  # Default value
-        try:
-           fontweight = dom.getitem('/window/terminal/tag/error/fontweight')
-        except XMLManagerError:
-           fontweight = 700  # Default value
-        self.TAG_DICT['error'] = [forecolor, backcolor, fontweight]
-
-        # Info tag
-
-        try:
-           forecolor = dom.getitem('/window/terminal/tag/info/forecolor')
-        except XMLManagerError:
-           forecolor = ''  # Default value
-        try:
-           backcolor = dom.getitem('/window/terminal/tag/info/backcolor')
-        except XMLManagerError:
-           backcolor = '#b0ffff'  # Default value
-        try:
-           fontweight = dom.getitem('/window/terminal/tag/info/fontweight')
-        except XMLManagerError:
-           fontweight = 400  # Default value
-        self.TAG_DICT['info'] = [forecolor, backcolor, fontweight]
-
-        # Line number tag 
-
-        try:
-           forecolor = dom.getitem('/window/terminal/tag/linenumber/forecolor')
-        except XMLManagerError:
-           forecolor = '#0000ff'  # Default value
-        try:
-           backcolor = dom.getitem('/window/terminal/tag/linenumber/backcolor')
-        except XMLManagerError:
-           backcolor = ''  # Default value
-        try:
-           fontweight = dom.getitem('/window/terminal/tag/linenumber/fontweight')
-        except XMLManagerError:
-           fontweight = 700  # Default value
-        self.TAG_DICT['linenumber'] = [forecolor, backcolor, fontweight]
-
-        # Note tag
-
-        try:
-           forecolor = dom.getitem('/window/terminal/tag/note/forecolor')
-        except XMLManagerError:
-           forecolor = '#8b008b'  # Default value
-        try:
-           backcolor = dom.getitem('/window/terminal/tag/note/backcolor')
-        except XMLManagerError:
-           backcolor = ''  # Default value
-        try:
-           fontweight = dom.getitem('/window/terminal/tag/note/fontweight')
-        except XMLManagerError:
-           fontweight = 400  # Default value
-        self.TAG_DICT['note'] = [forecolor, backcolor, fontweight]
-
-        # Warning tag
-
-        try:
-           forecolor = dom.getitem('/window/terminal/tag/warning/forecolor')
-        except XMLManagerError:
-           forecolor = ''  # Default value
-        try:
-           backcolor = dom.getitem('/window/terminal/tag/warning/backcolor')
-        except XMLManagerError:
-           backcolor = '#eeee80'  # Default value
-        try:
-           fontweight = dom.getitem('/window/terminal/tag/warning/fontweight')
-        except XMLManagerError:
-           fontweight = 400  # Default value
-        self.TAG_DICT['warning'] = [forecolor, backcolor, fontweight]
-
-        # Run Dialog window settings
-
-        try:
-           width = dom.getitem('/window/run_dialog/width')
-        except XMLManagerError:
-           width = 400   # Default value
-        try:
-           height = dom.getitem('/window/run_dialog/height')
-        except XMLManagerError:
-           height = 120   # Default value
-        self.run_dialog = WindowPreferences(width, height)
-        try:
-           history = dom.getitem('/window/run_dialog/history')
-        except XMLManagerError:
-           # Default value
-           history = ["",
-                      "emerge ",
-                      "ACCEPT_KEYWORDS='~x86' emerge ",
-                      "USE=' ' emerge ",
-                      "ACCEPT_KEYWORDS='~x86' USE=' ' emerge ",
-                      "emerge --help"]
-           default_history = len(history)
-        self.run_dialog.history = history
-        try:
-           default_history = dom.getitem('/window/run_dialog/default_history')
-        except XMLManagerError:
-           # Default value
-           # default_history = length of the history items to always remain
-           # at the start of the popdown history list & set above when history is set
-           default_history = len(history)
-        self.run_dialog.default_history = default_history
-        try:
-           history_length = dom.getitem('/window/run_dialog/history_length')
-        except XMLManagerError:
-           # Default value for maximum nuber of retained history items
-           history_length = 10
-        self.run_dialog.history_length = history_length
-
-        # Emerge options
- 
+        
+        taglist = [ \
+            ['default','','',400],
+            ['caution','','#c040b0',400],  # [name, default forecolor, backcolor, fontweight]
+            ['command','#ffffff','#000080',700],
+            ['emerge','','#90ee90',700],
+            ['error','#faf0e6','#ff0000',700],
+            ['info','','#b0ffff',400],
+            ['linenumber','#0000ff','',700],
+            ['note','#8b008b','',400],
+            ['warning','','#eeee80',400],
+            # the following tags are for emerge's output formatting
+            ['bold','','',700],
+            ['light','','',300],
+            ['fg_black','black','',None],
+            ['fg_red','darkred','',None],
+            ['fg_green','darkgreen','',None],
+            ['fg_yellow','brown','',None],
+            ['fg_blue','darkblue','',None],
+            ['fg_magenta','magenta','',None],
+            ['fg_cyan','blue','',None],
+            ['fg_white','yellow','',None],
+            ['bg_black','','black',None],
+            ['bg_red','','darkred',None],
+            ['bg_green','','darkgreen',None],
+            ['bg_yellow','','brown',None],
+            ['bg_blue','','darkblue',None],
+            ['bg_magenta','','magenta',None],
+            ['bg_cyan','','blue',None],
+            ['bg_white','','white',None],
+        ]
+        
+        for tag_name, forecolor, backcolor, fontweight in taglist:
+            try:
+                fc = dom.getitem(''.join(['/window/terminal/tag/',tag_name,'/forecolor']))
+            except XMLManagerError:
+                fc = forecolor
+            try:
+                bc = dom.getitem(''.join(['/window/terminal/tag/',tag_name,'/backcolor']))
+            except XMLManagerError:
+                bc = backcolor
+            try:
+                fw = dom.getitem(''.join(['/window/terminal/tag/',tag_name,'/fontweight']))
+            except XMLManagerError:
+                fw = fontweight
+            self.TAG_DICT[tag_name] = [fc, bc, fw]
+        
+        emergeoptions = ['pretend', 'fetch', 'verbose', 'upgradeonly', 'nospinner']
         self.emerge = EmergeOptions()
+        for option in emergeoptions:
+            try:
+                setattr(self.emerge, option, dom.getitem(''.join(['/emerge/options/', option])))
+            except XMLManagerError:
+                pass # defaults set in EmergeOptions class
+        
+        advemergeoptions = [
+            ['showuseflags', True],
+            ['showkeywords', True],
+            ['show_make_conf_button', False],
+        ]
+        self.advemerge = OptionsClass()
+        for option, default in advemergeoptions:
+            try:
+                value = dom.getitem(''.join(['/advemerge/', option]))
+            except XMLManagerError:
+                value = default
+            setattr(self.advemerge, option, value)
+        
+        viewoptions = [ \
+            ['downgradable_fg', '#FA0000'],
+            ['upgradable_fg', '#0000FF'],
+            ['normal_fg','#000000'],
+            ['normal_bg','#FFFFFF']
+        ]
+        self.views = OptionsClass()
+        for option, default in viewoptions:
+            try:
+                value = dom.getitem(''.join(['/views/', option]))
+            except XMLManagerError:
+                value = default
+            setattr(self.views, option, value)
+        
+        summaryoptions = [ \
+            ['showtable', True],
+            ['showkeywords', True],
+            ['showinstalled', True],
+            ['showavailable', True],
+            ['showlongdesc', True],
+            ['showuseflags', True],
+            ['showlicense', True],
+            ['showurl', True],
+            ['ebuilds_top', True],
+        ]
+        self.summary = OptionsClass()
+        for option, default in summaryoptions:
+            try:
+                value = dom.getitem(''.join(['/summary/', option]))
+            except XMLManagerError:
+                value = default
+            setattr(self.summary, option, value)
+        
+        # Misc. variables
+        
+        # probably depricated variables, was used for progressbar calc
         try:
-           self.emerge.pretend = dom.getitem('/emerge/options/pretend')
-        except XMLManagerError:
-           pass
-        try:
-           self.emerge.fetch = dom.getitem('/emerge/options/fetch')
-        except XMLManagerError:
-           pass
-        try:
-           self.emerge.verbose = dom.getitem('/emerge/options/verbose')
-        except XMLManagerError:
-           pass
-        try:
-           self.emerge.upgradeonly = dom.getitem('/emerge/options/upgradeonly')
-        except XMLManagerError:
-           pass
-        try:
-           self.emerge.nospinner = dom.getitem('/emerge/options/nospinner')
-        except XMLManagerError:
-           pass
-
-	# Misc. variables
-
-	try:
            self.database_size = dom.getitem('/database/size')
         except XMLManagerError:
            self.database_size = 7000
         try:
            self.dbtime = dom.getitem('/database/dbtime')
-	   #dprint("UTILS: __init__(); self.dbtime =")
-	   #dprint(self.dbtime)
+           #dprint("UTILS: __init__(); self.dbtime =")
+           #dprint(self.dbtime)
         except XMLManagerError:
            self.dbtime = 50
         try:
@@ -531,10 +431,74 @@ class PortholePreferences:
         except XMLManagerError:
            self.dbtotals = []
         
+        self.plugins = OptionsClass()
+        
+        globaloptions = [ \
+            ['LANG', 'en'],
+            ['enable_archlist', False],
+            ##['enable_all_keywords', False],
+            ["archlist", ["alpha", "amd64", "arm", "hppa", "ia64", "mips",
+                            "ppc", "ppc64", "s390", "sparc", "x86"]],
+            ["Sync", "emerge sync"],
+            ["Sync_label", _("Sync")],
+            #                use the form " [sync-command, sync-label],
+            # note: this is now hard-coded below
+            #["Sync_methods", [['emerge sync', _('Sync')],
+            #                  ['emerge-webrsync', _('WebRsync')],
+            #                  ['#user defined', _('Unknown Sync')]]],
+            ['custom_browser_command', 'firefox %s'],
+            ['use_custom_browser', False],
+            ['su', 'gksudo -g'], # -g tells gksu not to steal mouse / keyboard focus. Panics sometimes otherwise.
+        ]
+        
+        self.globals = OptionsClass()
+        for option, default in globaloptions:
+            try:
+                value = dom.getitem(''.join(['/globals/', option]))
+            except XMLManagerError:
+                value = default
+                dprint("DEFAULT VALUE: %s = %s" %(option,str(value)))
+            setattr(self.globals, option, value)
+            dprint("UTILS: PortholePreferences; setting globals.%s = %s" %(option, str(value)))
+        
+        if can_gksu(self.globals.su.split(' ')[0]) == False:
+            # If the current su option is not valid, try some others.
+            if can_gksu('gksudo'):
+                self.globals.su = 'gksudo -g'
+            elif can_gksu('gksu'):
+                self.globals.su = 'gksu -g'
+            elif can_gksu('gnomesu'):
+                self.globals.su = 'gnomesu'
+            elif can_gksu('kdesu'):
+                self.globals.su = 'kdesu'
+        
+        self.globals.Sync_methods = [['emerge sync', _('Sync')],
+                                     ['emerge-webrsync', _('WebRsync')]]
+        
+        # fix sync_label if translation changed
+        for method in self.globals.Sync_methods:
+            if method[0] == self.globals.Sync:
+                self.globals.Sync_label = method[1]
+        
+        if New_prefs:
+            for option, value in New_prefs:
+                setattr(self, option, value)
+        
+        
+        self.PLUGIN_DIR = self.DATA_PATH + 'plugins/' # could add more dirs later
+        dprint("UTILS: PortholePreferences; PLUGIN_DIR = %s" %self.PLUGIN_DIR)
+        self.plugins = OptionsClass()
+        try:
+            option = "active_list"
+            value = dom.getitem(''.join(['/plugins/', option]))
+        except XMLManagerError:
+            value = []
+        setattr(self.plugins, option, value)
+        dprint("UTILS: PortholePreferences; setting plugins.%s = %s" %(option, str(value)))
+
         # All prefs now loaded or defaulted
         del dom   # no longer needed, release memory
 
-
     def save(self):
         """ Save preferences """
         dprint("UTILS: preferences save()")
@@ -543,9 +507,12 @@ class PortholePreferences:
         dom.version = version
         dom.additem('/window/main/width', self.main.width)
         dom.additem('/window/main/height', self.main.height)
+        dom.additem('/window/main/xpos', self.main.xpos)
+        dom.additem('/window/main/ypos', self.main.ypos)
         dom.additem('/window/main/hpane', self.main.hpane)
         #dprint("UTILS: save() hpane: %d" %self.main.hpane)
         dom.additem('/window/main/vpane', self.main.vpane)
+        dom.additem('/window/main/maximized', self.main.maximized)
         dom.additem('/window/main/search_desc', self.main.search_desc)
         dom.additem('/window/main/show_nag_dialog', self.main.show_nag_dialog)
         dom.additem('/window/process/width', self.process.width)
@@ -554,6 +521,9 @@ class PortholePreferences:
         dom.additem('/window/terminal/width', self.terminal.width)
         dom.additem('/window/terminal/height', self.terminal.height)
         dom.additem('/window/terminal/width_verbose', self.terminal.width_verbose)
+        dom.additem('/window/terminal/font', self.terminal.font)
+        dom.additem('/window/terminal/all_tabs_use_custom_colors', \
+            self.terminal.all_tabs_use_custom_colors)
         # generate tag keys from dictionary
         for key in self.TAG_DICT:
             format_list = self.TAG_DICT[key] 
@@ -570,13 +540,43 @@ class PortholePreferences:
         dom.additem('/emerge/options/verbose', self.emerge.verbose)
         dom.additem('/emerge/options/upgradeonly', self.emerge.upgradeonly)
         dom.additem('/emerge/options/nospinner', self.emerge.nospinner)
-	dom.additem('/database/size', self.database_size)
-	#dprint("UTILS: save(); self.dbtime = %d" %self.dbtime)
-	dom.additem('/database/dbtime', self.dbtime)
-	dom.additem('/database/dbtotals', self.dbtotals)
+        dom.additem('/advemerge/showuseflags', self.advemerge.showuseflags)
+        dom.additem('/advemerge/showkeywords', self.advemerge.showkeywords)
+        dom.additem('/advemerge/show_make_conf_button', self.advemerge.show_make_conf_button)
+        dom.additem('/views/upgradable_fg', self.views.upgradable_fg)
+        dom.additem('/views/downgradable_fg', self.views.downgradable_fg)
+        dom.additem('/views/normal_fg', self.views.normal_fg)
+        dom.additem('/views/normal_bg', self.views.normal_bg)
+        dom.additem('/summary/showtable', self.summary.showtable)
+        dom.additem('/summary/showkeywords', self.summary.showkeywords)
+        dom.additem('/summary/showinstalled', self.summary.showinstalled)
+        dom.additem('/summary/showavailable', self.summary.showavailable)
+        dom.additem('/summary/showlongdesc', self.summary.showlongdesc)
+        dom.additem('/summary/showuseflags', self.summary.showuseflags)
+        dom.additem('/summary/showlicense', self.summary.showlicense)
+        dom.additem('/summary/showurl', self.summary.showurl)
+        dom.additem('/database/size', self.database_size)
+        #dprint("UTILS: save(); self.dbtime = %d" %self.dbtime)
+        dom.additem('/database/dbtime', self.dbtime)
+        dom.additem('/database/dbtotals', self.dbtotals)
+        #dom.additem('/plugins/path_list', self.plugins.path_list)
+        dom.additem('/globals/LANG', self.globals.LANG)
+        dom.additem('/globals/enable_archlist', self.globals.enable_archlist)
+        ##dom.additem('/globals/enable_all_keywords', self.globals.enable_all_keywords)
+        dom.additem('/globals/archlist', self.globals.archlist)
+        dom.additem('/globals/Sync', self.globals.Sync)
+        dom.additem('/globals/Sync_label', self.globals.Sync_label)
+        #dom.additem('/globals/Sync_methods', self.globals.Sync_methods)
+        dom.additem('/globals/custom_browser_command', self.globals.custom_browser_command)
+        dom.additem('/globals/use_custom_browser', self.globals.use_custom_browser)
+        dom.additem('/globals/su', self.globals.su)
+        dom.additem('/plugins/active_list', self.plugins.active_list)
         dom.save(self.__PFILE)
         del dom   # no longer needed, release memory
-
+    
+    def __repr__(self): # used by print statement (and pycrash)
+        """Return a string representation of the preferences"""
+        return self.__dict__.__repr__()
 
 class PortholeConfiguration:
     """ Holds all of Porthole's developer configurable settings """
@@ -585,49 +585,23 @@ class PortholeConfiguration:
 
         # Handle all the regular expressions.  They will be compiled
         # within this object for the sake of efficiency.
-
-        patternlist = dom.getitem('/re_filters/info')
-        self.info_re_list = []
-        for regexp in patternlist:
-            self.info_re_list.append(sre.compile(regexp))
-
-        patternlist = dom.getitem('/re_filters/notinfo')
-        self.info_re_notlist = []
-        for regexp in patternlist:
-            self.info_re_notlist.append(sre.compile(regexp))
-
-        patternlist = dom.getitem('/re_filters/warning')
-        self.warning_re_list = []
-        for regexp in patternlist:
-            self.warning_re_list.append(sre.compile(regexp))
-
-        patternlist = dom.getitem('/re_filters/notwarning')
-        self.warning_re_notlist = []
-        for regexp in patternlist:
-            self.warning_re_notlist.append(sre.compile(regexp))
-
-        patternlist = dom.getitem('/re_filters/error')
-        self.error_re_list = []
-        for regexp in patternlist:
-            self.error_re_list.append(sre.compile(regexp))
-
-        patternlist = dom.getitem('/re_filters/noterror')
-        self.error_re_notlist = []
-        for regexp in patternlist:
-            self.error_re_notlist.append(sre.compile(regexp))
-
-        patternlist = dom.getitem('/re_filters/caution')
-        self.caution_re_list = []
-        for regexp in patternlist:
-            self.caution_re_list.append(sre.compile(regexp))
-
-        patternlist = dom.getitem('/re_filters/notcaution')
-        self.caution_re_notlist = []
-        for regexp in patternlist:
-            self.caution_re_notlist.append(sre.compile(regexp))
-
+        
+        filterlist = ['info', 'warning', 'error', 'caution', 'needaction']
+        for filter in filterlist:
+            patternlist = dom.getitem(''.join(['/re_filters/',filter])) # e.g. '/re_filters/info'
+            attrname = ''.join([filter, '_re_list'])
+            setattr(self, attrname, []) # e.g. self.info_re_list = []
+            for regexp in patternlist:
+                getattr(self, attrname).append(sre.compile(regexp))
+            patternlist = dom.getitem(''.join(['/re_filters/not',filter])) # e.g. '/re_filters/notinfo'
+            attrname = ''.join([filter, '_re_notlist'])
+            setattr(self, attrname, []) # e.g. self.info_re_notlist = []
+            for regexp in patternlist:
+                getattr(self, attrname).append(sre.compile(regexp))
+        
         self.emerge_re = sre.compile(dom.getitem('/re_filters/emerge'))
         self.ebuild_re = sre.compile(dom.getitem('/re_filters/ebuild'))
+        self.merged_re = sre.compile(dom.getitem('/re_filters/merged'))
         del dom
 
     def isInfo(self, teststring):
@@ -652,7 +626,7 @@ class PortholeConfiguration:
         return False
 
     def isCaution(self, teststring):
-        ''' Parse string, return true if belongs in info tab '''
+        ''' Parse string, return true if matches caution regexp '''
         for regexp in self.caution_re_list:
             if regexp.match(teststring):
                 for regexpi in self.caution_re_notlist:
@@ -675,6 +649,21 @@ class PortholeConfiguration:
         ''' Parse string, return true if it is the initial emerge line '''
         return self.emerge_re.match(teststring) != None
 
+    def isMerged(self, teststring):
+        ''' Parse string, return true if it is the merged line '''
+        return self.merged_re.search(teststring) != None
+    def isAction(self, teststring):
+        '''
+        Returns True if teststring matches the pre-set criteria for notification of an
+        action the user is recommended to take, such as etc-update or revdep-rebuild.
+        '''
+        for regexp in self.needaction_re_list:
+            if regexp.match(teststring):
+                for regexpi in self.needaction_re_notlist:
+                    if regexpi.match(teststring):
+                        return False    # excluded, no match
+                return True
+        return False
 
 class BadLogFile(Exception):
     """ Raised when we encounter errors parsing the log file."""
@@ -724,16 +713,15 @@ def estimate(package_name, log_file_name="/var/log/emerge.log"):
                         end_time = string.atof((tokens[0])[0:-1])
                         emerge_count += 1
                         total_time = total_time +\
-								(datetime.datetime.fromtimestamp(end_time) -
-                                 datetime.datetime.fromtimestamp(start_time))
+                                (datetime.datetime.fromtimestamp(end_time) -
+                                datetime.datetime.fromtimestamp(start_time))
                         break
         if emerge_count > 0:
             return total_time / emerge_count
         else:
             return None          
     except:
-        raise BadLogFile, _("Error reading emerge log file.  Check file " +\
-                          "permissions, or check for corrupt log file.")   
+        raise BadLogFile, _("Error reading emerge log file.  Check file permissions, or check for corrupt log file.")
 
 def pretend_check(command_string):
     isPretend = (sre.search("--pretend", command_string) != None)
@@ -751,3 +739,7 @@ def pretend_check(command_string):
 
 def help_check(command_string):
     return (sre.search("--help", command_string) != None)
+
+def info_check(command_string):
+    return (sre.search("emerge info", command_string) != None)
+
